"""UI components for MCP servers.

Provides Rich-based terminal UI components using Oneiric console.
"""

from __future__ import annotations

from .panels import ServerPanels

__all__ = ["ServerPanels"]
